function sfun_pendelstangeanim(block)
  % S-Funktion für die Animation eines Fadenpendels mit Stange
  % Eingang: Winkel phi
  % Parameter: phiS, l, lv
  setup(block);
end
  
%% -------------------------------------------------------------------
function setup(block)
  % Zahl der Anschlüsse
  block.NumInputPorts  = 1;
  block.NumOutputPorts = 0;
  block.NumDialogPrms  = 3;

  block.SetPreCompInpPortInfoToDynamic;  % Datentyp übernehmen
  block.SampleTimes = [-1 0];            % Samplezeit übernehmen

  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);
  block.RegBlockMethod('Start', @Start);
  block.RegBlockMethod('Update', @Update);
end

%% -------------------------------------------------------------------
function DoPostPropSetup(block)
  % Zeiger auf die Figur und die variablen Zeichenobjekte (Faden, Kugel)
  block.NumDworks = 1;
  
  block.Dwork(1).Name = 'handles'; 
  block.Dwork(1).Dimensions      = 3;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = false;
end

%% -------------------------------------------------------------------
function Start(block)
  % Parameterwerte
  phiS = block.DialogPrm(1).Data;
  l    = block.DialogPrm(2).Data;
  lv   = block.DialogPrm(3).Data;

  % erzeugt die Figur incl. Zeichenobjekten
  hFigur = initFrame('Pendel-Animation');
  lim = 1.2;
  plot([lim lim -lim -lim lim], [-lim lim lim -lim -lim], 'k-')
  set(gca, 'XTickMode', 'manual', 'YTickMode', 'manual');   
  axis([-lim lim -lim lim]);
  hold on;
  
  % Halterung und Stange, fix
  sH = 0.05;
  plot([sH, sH, -sH, -sH, sH], [-sH, sH, sH, -sH, -sH]);
  xS =  (l - lv)*sin(phiS);
  yS = -(l - lv)*cos(phiS);
  [xSK ySK] = computeCircle(0.02);
  plot(xS + xSK, yS + ySK);
  
  % Faden
  hFaden = plot([0, 0], [0, -1]);
  
  % Kugel
  [xK yK] = computeCircle(0.1);
  hKugel = patch(xK, yK - 1, [0 0 0]);
    
  % speichere Werte für Update
  block.Dwork(1).Data = [hFigur, hFaden, hKugel];
end

%% -------------------------------------------------------------------
function Update(block)
  % get values from work space
  hFigur = block.Dwork(1).Data(1);
  hFaden = block.Dwork(1).Data(2);
  hKugel = block.Dwork(1).Data(3);
  
  % get values from input and parameters
  phi  = block.InputPort(1).Data;
  phiS = block.DialogPrm(1).Data;
  l    = block.DialogPrm(2).Data;
  lv   = block.DialogPrm(3).Data;
  
  % prüfen, ob das Plotfenster während des Laufs geschlossen wurde
  if any(get(0, 'Children') == hFigur)  % gibt es die Figur noch?
    if strcmp(get(hFigur, 'Name'), 'Pendel-Animation') 
      % check: könnte ja inzwischen ein anderes Fenster sein!
      
      if (phi > phiS)
        x =  l*sin(phi);
        y = -l*cos(phi);
        xFaden = [0 x];
        yFaden = [0 y];
      else
        xS =  (l - lv)*sin(phiS);
        yS = -(l - lv)*cos(phiS);
        x = xS + lv*sin(phi);
        y = yS -lv*cos(phi);
        xFaden = [0, xS, x];
        yFaden = [0, yS, y];          
      end
      
      set(hFaden,'XData', xFaden);
      set(hFaden,'YData', yFaden);

      [xK yK] = computeCircle(0.1);
      set(hKugel,'XData', xK + x);
      set(hKugel,'YData', yK + y);

      drawnow;
    end
  end
end

%% -------------------------------------------------------------------
function hFigur = initFrame(name)
  % erzeuge Figur, falls nötig
  
  hFigur = findobj('Type','figure', 'Name', name)';
  if isempty(hFigur)
    hFigur = figure('Name', name, 'NumberTitle','off', 'MenuBar', 'none');

    % setze Größe, aber lasse die aktuelle Position auf dem Schirm
    position = get(hFigur, 'Position');
    position(3:4) = [300 300];
    set(hFigur, 'Position', position);
  
    % positioniere Achsen innerhalb der Figur (großer Rand soll weg)
    axes('Position',[0.05 0.05 0.9 0.9]);
  else
    figure(hFigur)           % bringe Figur nach vorne
  end

  cla reset;       % wichtig bei Restart der Simulation
     % Achtung: setzt axis-Eigenschaften auf Defaults, daher erst hier:
  set(gca,'DrawMode','fast');
  axis off;
end

%% -------------------------------------------------------------------
function [xK yK] = computeCircle(r)
  % berechnet Basiskoordinaten für die Pendelkugel
  N = 24;   % Zahl der Ecken
  t = 0:2*pi/N:2*pi;
  xK = r*cos(t);
  yK = r*sin(t);
end
